@@ -287,7 +287,7 @@ class ImageMount:
 
                 size = partition_info[3].split("B")[0]
 
-                if len(partition_info) < 6:
+                if len(partition_info) < 6 or partition_info[5] in ["boot"]:
                     # No filesystem can be found from partition line. Assuming
                     # btrfs, because that is the only MeeGo fs that parted does 
                     # not recognize properly.
