@@ -364,7 +364,7 @@ class ImageCreator(object):
         A sensible default implementation is provided.
 
         """
-        s =  "/dev/root  /         %s    defaults,noatime 0 0\n" %(self._fstype)
+        s =  "/dev/root  /         %s    %s 0 0\n" % (self._fstype, "defaults,noatime" if not self._fsopts else self._fsopts)
         s += self._get_fstab_special()
         return s
 
@@ -800,11 +800,14 @@ class ImageCreator(object):
         for repo in kickstart.get_repos(self.ks, repo_urls):
             (name, baseurl, mirrorlist, inc, exc, proxy, proxy_username, proxy_password, debuginfo, source, gpgkey, disable) = repo
 
-            yr = pkg_manager.addRepository(name, baseurl, mirrorlist, proxy, proxy_username, proxy_password)
-            if inc:
-                yr.includepkgs = inc
-            if exc:
-                yr.exclude = exc
+            try:
+                yr = pkg_manager.addRepository(name, baseurl, mirrorlist, proxy, proxy_username, proxy_password, inc, exc)
+                if inc:
+                    yr.includepkgs = inc
+                if exc:
+                    yr.exclude = exc
+            except CreatorError, e:
+                raise CreatorError("%s" % (e,))
 
         if kickstart.exclude_docs(self.ks):
             rpm.addMacro("_excludedocs", "1")
@@ -813,21 +816,28 @@ class ImageCreator(object):
             rpm.addMacro("_install_langs", kickstart.inst_langs(self.ks))
 
         try:
-            try:
-                self.__select_packages(pkg_manager)
-                self.__select_groups(pkg_manager)
-                self.__deselect_packages(pkg_manager)
-                self.__localinst_packages(pkg_manager)
+            self.__select_packages(pkg_manager)
+            self.__select_groups(pkg_manager)
+            self.__deselect_packages(pkg_manager)
+            self.__localinst_packages(pkg_manager)
 
-                BOOT_SAFEGUARD = 256L * 1024 * 1024 # 256M
-                checksize = self._root_fs_avail
-                if checksize:
-                    checksize -= BOOT_SAFEGUARD
-                if self.target_arch:
-                    pkg_manager._add_prob_flags(rpm.RPMPROB_FILTER_IGNOREARCH)
-                pkg_manager.runInstall(checksize)
-            except CreatorError, e:
-                raise CreatorError("%s" % (e,))
+            BOOT_SAFEGUARD = 256L * 1024 * 1024 # 256M
+            checksize = self._root_fs_avail
+            if checksize:
+                checksize -= BOOT_SAFEGUARD
+            if self.target_arch:
+                pkg_manager._add_prob_flags(rpm.RPMPROB_FILTER_IGNOREARCH)
+
+            try:
+                save_env = os.environ["LC_ALL"]
+            except KeyError:
+                save_env = None
+            os.environ["LC_ALL"] = 'C'
+            pkg_manager.runInstall(checksize)
+            if save_env:
+                os.environ["LC_ALL"] = save_env
+            else:
+                os.unsetenv("LC_ALL")
         finally:
             if keep_record:
                 self._pkgs_content = pkg_manager.getAllContent()
@@ -1115,8 +1125,10 @@ class LoopImageCreator(ImageCreator):
         self.__blocksize = 4096
         if self.ks:
             self.__fstype = kickstart.get_image_fstype(self.ks, "ext3")
+            self.__fsopts = kickstart.get_image_fsopts(self.ks, "defaults,noatime")
         else:
             self.__fstype = None
+            self.__fsopts = None
 
         self.__instloop = None
         self.__imgdir = None
@@ -1215,6 +1227,17 @@ class LoopImageCreator(ImageCreator):
 
     """
 
+    def __get_fsopts(self):
+        return self.__fsopts
+    def __set_fsopts(self, val):
+        self.__fsopts = val
+    _fsopts = property(__get_fsopts, __set_fsopts)
+    """Mount options of filesystem used for the image.
+
+    This can be specified by --fsoptions="xxx,yyy" in part command in
+    kickstart file.
+    """
+
     #
     # Helpers for subclasses
     #
@@ -1277,6 +1300,16 @@ class LoopImageCreator(ImageCreator):
 
 
 class FsImageCreator(ImageCreator):
+    def __init__(self, ks, name):
+        """Initialize a LoopImageCreator instance.
+
+        This method takes the same arguments as ImageCreator.__init__()
+        """
+        ImageCreator.__init__(self, ks, name)
+
+        self._fstype = None
+        self._fsopts = None
+
     def _stage_final_image(self):
         """ nothing to do """
         pass
@@ -1288,21 +1321,16 @@ class FsImageCreator(ImageCreator):
         if self._recording_pkgs:
             self._save_recording_pkgs(destdir)
 
-        print "Moving %s to %s, please be patient to wait (it is slow if they are on different file systems/partitons/disks)" \
+        print "Copying %s to %s, please be patient to wait (it is slow if they are on different file systems/partitons/disks)" \
                % (self._instroot, destdir + "/" + self.name)
 
         copycmd = find_binary_path("cp")
-        args = [ copycmd, "-af", self._instroot + "/dev", os.path.dirname(self._instroot) ]
+        args = [ copycmd, "-af", self._instroot, destdir + "/" + self.name ]
         subprocess.call(args)
-        shutil.rmtree(self._instroot + "/dev", ignore_errors = True)
-
-        shutil.move(self._instroot, destdir + "/" + self.name)
 
         ignores = ["/dev/fd", "/dev/stdin", "/dev/stdout", "/dev/stderr", "/etc/mtab"]
         for exclude in ignores:
             if os.path.exists(destdir + "/" + self.name + exclude):
                 os.unlink(destdir + "/" + self.name + exclude)
 
-        args = [ copycmd, "-af", os.path.dirname(self._instroot) + "/dev", destdir + "/" + self.name ]
-        subprocess.call(args)
         self.outimage.append(destdir + "/" + self.name)
