@@ -604,6 +604,12 @@ def get_image_fstype(ks, default = None):
             return p.fstype
     return default
 
+def get_image_fsopts(ks, default = None):
+    for p in ks.handler.partition.partitions:
+        if p.mountpoint == "/" and p.fsopts:
+            return p.fstype
+    return default
+
 def get_modules(ks):
     devices = []
     if isinstance(ks.handler.device, kscommands.device.FC3_Device):
@@ -762,16 +768,41 @@ def remove_duplicate_repos(ks):
             i += 1
             j = i + 1
 
-def support_patterns(ks, repometadata, use_comps = False):
+def resolve_groups(creator, repometadata, use_comps = False):
+    pkgmgr = creator.pkgmgr.get_default_pkg_manager()
+    iszypp = False
+    if creator.pkgmgr.managers.has_key("zypp") and creator.pkgmgr.managers['zypp'] == pkgmgr:
+        iszypp = True
+    ks = creator.ks
+
     for repo in repometadata:
-        if repo["comps"] and use_comps:
+        """ Mustn't replace group with package list if repo is ready for the corresponding package manager """
+        if iszypp and repo["patterns"] and not use_comps:
             continue
-        if repo["patterns"]:
+        if not iszypp and repo["comps"] and use_comps:
+            continue
+
+        """
+            But we also must handle such cases, use zypp but repo only has comps, 
+            use yum but repo only has patterns, use zypp but use_comps is true,
+            use yum but use_comps is false.
+        """
+        groupfile = None
+        if iszypp:
+            if (use_comps and repo["comps"]) or (not repo["patterns"] and repo["comps"]):
+                groupfile = repo["comps"]
+                get_pkglist_handler = misc.get_pkglist_in_comps
+        if not iszypp:
+            if (not use_comps and repo["patterns"]) or (not repo["comps"] and repo["patterns"]):
+                groupfile = repo["patterns"]
+                get_pkglist_handler = misc.get_pkglist_in_patterns
+            
+        if groupfile:
             i = 0
             while True:
                 if i >= len(ks.handler.packages.groupList):
                     break
-                pkglist = misc.get_pkglist_in_group(ks.handler.packages.groupList[i].name, repo["patterns"])
+                pkglist = get_pkglist_handler(ks.handler.packages.groupList[i].name, groupfile)
                 if pkglist:
                     del ks.handler.packages.groupList[i]
                     for pkg in pkglist:
