@@ -57,6 +57,8 @@ class Zypp:
         self.ts = None
         self.probFilterFlags = []
         self.bin_rpm = find_binary_path("rpm")
+        self.incpkgs = []
+        self.excpkgs = []
 
     def doFileLogSetup(self, uid, logfile):
         # don't do the file log for the livecd as it can lead to open fds
@@ -112,22 +114,53 @@ class Zypp:
         self.installroot = installroot
 
     def selectPackage(self, pkg):
-        """Select a given package.  Can be specified with name.arch or name*"""
+        """ Select a given package or package pattern, can be specified with name.arch or name* or *name """
         if not self.Z:
             self.__initialize_zypp()
-
+        
         found = False
+        startx = pkg.startswith("*")
+        endx = pkg.endswith("*")
+        ispattern = startx or endx
+        sp = pkg.rsplit(".", 2)
         for item in self.Z.pool():
             kind = "%s" % item.kind()
             if kind == "package":
                 name = "%s" % item.name()
-                if name == pkg:
-                    found = True
-                    if name not in self.packages:
-                        self.packages.append(name)
-                        item.status().setToBeInstalled (zypp.ResStatus.USER)
-                    break
+                if not ispattern:
+                    if name in self.incpkgs or self.excpkgs:
+                        found = True
+                        break
+                    if len(sp) == 2:
+                        arch = "%s" % item.arch()
+                        if name == sp[0] and arch == sp[1]:
+                            found = True
+                            if name not in self.packages:
+                                self.packages.append(name)
+                                item.status().setToBeInstalled (zypp.ResStatus.USER)
+                            break
+                    else:
+                        if name == sp[0]:
+                            found = True
+                            if name not in self.packages:
+                                self.packages.append(name)
+                                item.status().setToBeInstalled (zypp.ResStatus.USER)
+                            break
+                else:
+                    if name in self.incpkgs or self.excpkgs:
+                        found =  True
+                        continue
+                    if startx and name.endswith(sp[0][1:]):
+                        found = True
+                        if name not in self.packages:
+                            self.packages.append(name)
+                            item.status().setToBeInstalled (zypp.ResStatus.USER)
 
+                    if endx and name.startswith(sp[0][:-1]):
+                        found = True
+                        if name not in self.packages:
+                            self.packages.append(name)
+                            item.status().setToBeInstalled (zypp.ResStatus.USER)
         if found:
             return None
         else:
@@ -135,23 +168,83 @@ class Zypp:
             return e
 
     def deselectPackage(self, pkg):
+        """Deselect package.  Can be specified as name.arch or name*"""
+        
         if not self.Z:
             self.__initialize_zypp()
-
-        """Deselect package.  Can be specified as name.arch or name*"""
+        
+        startx = pkg.startswith("*")
+        endx = pkg.endswith("*")
+        ispattern = startx or endx
         sp = pkg.rsplit(".", 2)
         for item in self.Z.pool():
             kind = "%s" % item.kind()
             if kind == "package":
                 name = "%s" % item.name()
-                if name == sp[0]:
-                    if item.status().isInstalled():
-                        item.status().setToBeUninstalled (zypp.ResStatus.USER)
-                    if name in self.packages:
-                        self.packages.remove(name)
-                    break
-        
+                if not ispattern:
+                    if len(sp) == 2:
+                        arch = "%s" % item.arch()
+                        if name == sp[0] and arch == sp[1]:
+                            if item.status().isToBeInstalled():
+                                item.status().resetTransact(zypp.ResStatus.USER)
+                            if name in self.packages:
+                                self.packages.remove(name)
+                            break
+                    else:
+                        if name == sp[0]:
+                            if item.status().isToBeInstalled():
+                                item.status().resetTransact(zypp.ResStatus.USER)
+                            if name in self.packages:
+                                self.packages.remove(name)
+                            break                             
+                else:
+                    if startx and name.endswith(sp[0][1:]):
+                        if item.status().isToBeInstalled():
+                            item.status().resetTransact(zypp.ResStatus.USER)
+                        if name in self.packages:
+                            self.packages.remove(name)
 
+                    if endx and name.startswith(sp[0][:-1]):
+                        if item.status().isToBeInstalled():
+                            item.status().resetTransact(zypp.ResStatus.USER)
+                        if name in self.packages:
+                            self.packages.remove(name)
+    
+    def __selectIncpkgs(self):        
+        found = False
+        for pkg in self.incpkgs:
+            for item in self.Z.pool():
+                kind = "%s" % item.kind()
+                if kind == "package":
+                    name = "%s" % item.name()
+                    repoalias = "%s" % item.repoInfo().alias()
+                    if name == pkg and repoalias.endswith("include"):
+                        found = True
+                        if name not in self.packages:
+                            self.packages.append(name)
+                            item.status().setToBeInstalled (zypp.ResStatus.USER)
+                        break         
+        if not found:
+            raise CreatorError("Unable to find package: %s" % (pkg,))
+    
+    def __selectExcpkgs(self):    
+        found = False        
+        for pkg in self.excpkgs:
+            for item in self.Z.pool():
+                kind = "%s" % item.kind()
+                if kind == "package":
+                    name = "%s" % item.name()
+                    repoalias = "%s" % item.repoInfo().alias()
+                    if name == pkg and not repoalias.endswith("exclude"):
+                        found = True
+                        if name not in self.packages:
+                            self.packages.append(name)
+                            item.status().setToBeInstalled (zypp.ResStatus.USER)
+                        break                     
+        if not found:
+            raise CreatorError("Unable to find package: %s" % (pkg,))
+
+        
     def selectGroup(self, grp, include = pykickstart.parser.GROUP_DEFAULT):
         if not self.Z:
             self.__initialize_zypp()
@@ -333,7 +426,7 @@ class Zypp:
         shutil.rmtree(repo_lic_dir)
         return True
 
-    def addRepository(self, name, url = None, mirrorlist = None, proxy = None, proxy_username = None, proxy_password = None):
+    def addRepository(self, name, url = None, mirrorlist = None, proxy = None, proxy_username = None, proxy_password = None, inc = None, exc = None):
         if not self.repo_manager:
             self.__initialize_repo_manager()
 
@@ -344,6 +437,13 @@ class Zypp:
         repo.proxy_username = proxy_username
         repo.proxy_password = proxy_password
         repo.baseurl.append(url)
+        repo_alias = repo.id
+        if inc:
+            repo_alias = name + "include"
+            self.incpkgs = inc
+        if exc:
+            repo_alias = name + "exclude"
+            self.excpkgs = exc
 
         # check LICENSE files
         if not self.checkRepositoryEULA(name, repo):
@@ -357,15 +457,18 @@ class Zypp:
         self.repos.append(repo)
 
 
-        repo_info = zypp.RepoInfo()
-        repo_info.setAlias(repo.id)
-        repo_info.setName(repo.name)
-        repo_info.setEnabled(repo.enabled)
-        repo_info.setAutorefresh(repo.autorefresh)
-        repo_info.setKeepPackages(repo.keeppackages)
-        repo_info.addBaseUrl(zypp.Url(repo.baseurl[0]))
-        self.repo_manager.addRepository(repo_info)
-        self.__build_repo_cache(name)
+        try:
+            repo_info = zypp.RepoInfo()
+            repo_info.setAlias(repo_alias)
+            repo_info.setName(repo.name)
+            repo_info.setEnabled(repo.enabled)
+            repo_info.setAutorefresh(repo.autorefresh)
+            repo_info.setKeepPackages(repo.keeppackages)
+            repo_info.addBaseUrl(zypp.Url(repo.baseurl[0]))
+            self.repo_manager.addRepository(repo_info)
+            self.__build_repo_cache(name)
+        except RuntimeError, e:
+            raise CreatorError("%s" % (e,))
 
         return repo
 
@@ -373,6 +476,11 @@ class Zypp:
         return False
 
     def runInstall(self, checksize = 0):
+        if self.incpkgs:
+            self.__selectIncpkgs()
+        if self.excpkgs:
+            self.__selectExcpkgs()
+        
         os.environ["HOME"] = "/"
         self.buildTransaction()
 
@@ -507,7 +615,7 @@ class Zypp:
         if not self.Z.resolver().resolvePool():
             print "Problem count: %d" % len(self.Z.resolver().problems())
             for problem in self.Z.resolver().problems():
-                print "Problem: %s, %s" % (problem.description(), problem.details())
+                print "Problem: %s, %s" % (problem.description().decode("utf-8"), problem.details().decode("utf-8"))
 
     def getLocalPkgPath(self, po):
         repoinfo = po.repoInfo()
@@ -547,8 +655,12 @@ class Zypp:
             proxies = {}
             if proxy:
                 proxies = {str(proxy.split(":")[0]):str(proxy)}
-            
-            url = baseurl + "/%s/%s" % (po.arch(), os.path.basename(filename))
+          
+            location = zypp.asKindPackage(po).location()
+            location = location.filename().__str__()
+            if location.startswith("./"):
+                location = location[2:]
+            url = baseurl + "/%s" % location
             try:
                 filename = myurlgrab(url, filename, proxies)
             except CreatorError, e:
@@ -572,6 +684,12 @@ class Zypp:
                 rpmpath = self.localpkgs[pkgname]
             else:
                 rpmpath = self.getLocalPkgPath(po)
+            if not os.path.exists(rpmpath):
+                """ Maybe it is a local repo """
+                baseurl = po.repoInfo().baseUrls()[0].__str__()
+                baseurl = baseurl.strip()
+                if baseurl.startswith("file:/"):
+                    rpmpath = baseurl[5:] + "/%s/%s" % (po.arch(), os.path.basename(rpmpath))
             if not os.path.exists(rpmpath):
                 raise RpmError("Error: %s doesn't exist" % rpmpath)
             h = readRpmHeader(self.ts, rpmpath)
