@@ -89,7 +89,6 @@ def get_repo_metadata(repo, cachedir, reponame, proxies = {}, arch = None):
             primarydb = primarydb[0:len(primarydb)-4]
     return primarydb
 
-
 def gen_sqlitedb_from_xml(primaryxml, sqlitedb, target_arch = None):
     con = sqlite.connect(sqlitedb)
     con.executescript("""
@@ -105,85 +104,81 @@ def gen_sqlitedb_from_xml(primaryxml, sqlitedb, target_arch = None):
     
     sql = "CREATE TABLE IF NOT EXISTS requires (  name TEXT,  flags TEXT,  epoch TEXT,  version TEXT,  release TEXT,  pkgKey INTEGER , pre BOOLEAN DEFAULT FALSE);"
     con.execute(sql)
-    
-    dom = xml.dom.minidom.parse(primaryxml)
-    
+
+    elementroot = xmlparse(primaryxml).getroot()
     pkgKey = 0
-    nodelist = dom.getElementsByTagName("package")
-    for node in nodelist:
-        if (node.nodeType == node.ELEMENT_NODE) and (len(node.childNodes) >= 12):
+    
+    pkgelements = elementroot.getchildren()
+    for element in pkgelements:
+        if (len(list(element)) >= 12):
             pkgKey += 1
             myprovides = []
             myrequires = []
-            for subnode in node.childNodes:
-                if (subnode.nodeType != subnode.ELEMENT_NODE):
-                    continue
-                if (subnode.nodeName == "name"):
-                    pkgname = subnode.childNodes[0].data
-                elif (subnode.nodeName == "arch"):
-                    arch = subnode.childNodes[0].data
-                elif (subnode.nodeName == "version"):
-                    epoch = subnode.getAttribute("epoch")
-                    version = subnode.getAttribute("ver")
-                    release = subnode.getAttribute("rel")
-                elif (subnode.nodeName == "checksum"):
-                    pkgId = subnode.childNodes[0].data
-                    checksum_type = subnode.getAttribute("type")
-                elif (subnode.nodeName == "summary"):
-                    summary = subnode.childNodes[0].data
-                elif (subnode.nodeName == "description"):
-                    if subnode.childNodes:
-                        description = subnode.childNodes[0].data
+            for subelement in element.getchildren():
+                if subelement.tag.endswith("name"):
+                    pkgname = subelement.text
+                elif subelement.tag.endswith("arch"):
+                    arch = subelement.text
+                elif subelement.tag.endswith("version"):
+                    epoch = subelement.get("epoch", "")
+                    version = subelement.get("ver", "")
+                    release = subelement.get("rel", "")
+                elif subelement.tag.endswith("checksum"):
+                    pkgId = subelement.text
+                    checksum_type = subelement.get("type", "")
+                elif subelement.tag.endswith("summary"):
+                    summary = subelement.text
+                elif subelement.tag.endswith("description"):
+                    if subelement.text:
+                        description = subelement.text
                     else:
                         description = summary
-                elif (subnode.nodeName == "packager"):
-                    if len(subnode.childNodes) != 0:
-                        rpm_packager = subnode.childNodes[0].data
+                elif subelement.tag.endswith("packager"):
+                    if subelement.text:
+                        rpm_packager = subelement.text
                     else:
                         rpm_packager = ""
-                elif (subnode.nodeName == "url"):
-                    if len(subnode.childNodes) != 0:
-                        url = subnode.childNodes[0].data
+                elif subelement.tag.endswith("url"):
+                    if subelement.text:
+                        url = subelement.text
                     else:
                         url = ""
-                elif (subnode.nodeName == "time"):
-                    time_file = subnode.getAttribute("file")
-                    time_build = subnode.getAttribute("build")
-                elif (subnode.nodeName == "size"):
-                    size_package = subnode.getAttribute("package")
-                    size_installed = subnode.getAttribute("installed")
-                    size_archive = subnode.getAttribute("archive")
-                elif (subnode.nodeName == "location"):
-                    location_href = subnode.getAttribute("href")
-                    location_base = subnode.getAttribute("base")
-                elif (subnode.nodeName == "format"):
-                    for subsubnode in subnode.childNodes:
-                        if (subnode.nodeType != subnode.ELEMENT_NODE):
-                            continue
-                        if (subsubnode.nodeName == "rpm:license"):
-                            rpm_license = subsubnode.childNodes[0].data
-                        elif (subsubnode.nodeName == "rpm:vendor"):
-                            rpm_vendor = subsubnode.childNodes[0].data
-                        elif (subsubnode.nodeName == "rpm:group"):
-                            rpm_group = subsubnode.childNodes[0].data
-                        elif (subsubnode.nodeName == "rpm:buildhost"):
-                            rpm_buildhost = subsubnode.childNodes[0].data
-                        elif (subsubnode.nodeName == "rpm:sourcerpm"):
-                            if (len(subsubnode.childNodes) != 0):
-                                rpm_sourcerpm = subsubnode.childNodes[0].data
+                elif subelement.tag.endswith("time"):
+                    time_file = subelement.get("file", "")
+                    time_build = subelement.get("build", "")
+                elif subelement.tag.endswith("size"):
+                    size_package = subelement.get("package", "")
+                    size_installed = subelement.get("installed", "")
+                    size_archive = subelement.get("archive", "")
+                elif subelement.tag.endswith("location"):
+                    location_href = subelement.get("href", "")
+                    location_base = subelement.get("base", "")
+                elif subelement.tag.endswith("format"):
+                    for subsubelement in subelement.getchildren():
+                        if subsubelement.tag.endswith("license"):
+                            rpm_license = subsubelement.text
+                        elif subsubelement.tag.endswith("vendor"):
+                            rpm_vendor = subsubelement.text
+                        elif subsubelement.tag.endswith("group"):
+                            rpm_group = subsubelement.text
+                        elif subsubelement.tag.endswith("buildhost"):
+                            rpm_buildhost = subsubelement.text
+                        elif subsubelement.tag.endswith("sourcerpm"):
+                            if subsubelement.text:
+                                rpm_sourcerpm = subsubelement.text
                             else:
                                 rpm_sourcerpm = ""
-                        elif (subsubnode.nodeName == "rpm:header-range"):
-                            rpm_header_start = subsubnode.getAttribute("start")
-                            rpm_header_end = subsubnode.getAttribute("end")
-                        elif (subsubnode.nodeName == "rpm:provides"):
-                            for tmpnpode in subsubnode.childNodes:
-                                if (tmpnpode.nodeType == tmpnpode.ELEMENT_NODE) and (tmpnpode.nodeName == "rpm:entry"):
-                                    myprovides.append((tmpnpode.getAttribute("name"), tmpnpode.getAttribute("flag"), tmpnpode.getAttribute("epoch"), tmpnpode.getAttribute("ver"), tmpnpode.getAttribute("rel"), pkgKey))
-                        elif (subsubnode.nodeName == "rpm:requires"):
-                            for tmpnpode in subsubnode.childNodes:
-                                if (tmpnpode.nodeType == tmpnpode.ELEMENT_NODE) and (tmpnpode.nodeName == "rpm:entry"):
-                                    myrequires.append((tmpnpode.getAttribute("name"), tmpnpode.getAttribute("flag"), tmpnpode.getAttribute("epoch"), tmpnpode.getAttribute("ver"), tmpnpode.getAttribute("rel"), pkgKey, tmpnpode.getAttribute("pre")))
+                        elif subsubelement.tag.endswith("header-range"):
+                            rpm_header_start = subsubelement.get("start", "")
+                            rpm_header_end = subsubelement.get("end", "")
+                        elif subsubelement.tag.endswith("provides"):
+                            for tmpelement in subsubelement.getchildren():
+                                if tmpelement.tag.endswith("entry"):
+                                    myprovides.append((tmpelement.get("name", ""), tmpelement.get("flag", ""), tmpelement.get("epoch", ""), tmpelement.get("ver", ""), tmpelement.get("rel", ""), pkgKey))
+                        elif subsubelement.tag.endswith("requires"):
+                            for tmpelement in subsubelement.getchildren():
+                                if tmpelement.tag.endswith("entry"):
+                                    myrequires.append((tmpelement.get("name", ""), tmpelement.get("flag", ""), tmpelement.get("epoch", ""), tmpelement.get("ver", ""), tmpelement.get("rel", ""), pkgKey, tmpelement.get("pre", "")))
             """ Skip arm packages on ia32 """
             if not target_arch and arch.startswith("arm"):
                 continue
@@ -210,10 +205,10 @@ def gen_sqlitedb_from_xml(primaryxml, sqlitedb, target_arch = None):
             con.executemany(sql, myprovides)
             sql = "INSERT INTO requires values (?,?,?,?,?,?,?)";
             con.executemany(sql, myrequires)
-        
     con.commit()
     con.close()
 
+
 def get_my_all_deps_in_order(pkg, con):
     class Node:
         def __init__(self, data=None):
@@ -345,7 +340,7 @@ def install_rpm(pkgpath, pkgname, rootpath, force = False):
     rpm = find_binary_path("rpm")
 
     rootpath = os.path.abspath(os.path.expanduser(rootpath))
-    print "Installing %s from %s to %s..." % (pkgname,pkgpath,rootpath)
+    print "Installing %s..." % os.path.basename(pkgpath)
     dev_null = os.open("/dev/null", os.O_WRONLY)
     try:
         argv = [rpm, "-i", "--ignorearch", "--ignoreos", "--nodigest", "--nosignature", "--root=" + rootpath, pkgpath]
@@ -409,7 +404,10 @@ def build_bootstrap(repoList, mainrepo, cachedir, bootstrapdir, bootstrap_arch =
     
     """ HACK: This hack is here to enable backward compatibility until the 
         .ks files have been updated to support the @ARCH@ option for the URLs."""
-    if bootstrap_target_arch and bootstrap_target_arch.startswith("arm") and repourl.find(bootstrap_target_arch) != -1:
+    """ We can't use repourl.find(bootstrap_target_arch) != -1 here because
+        for example armv7nhl architecture uses armv7hl repository URL. 
+        This is similar to iX86 uses ia32 repository URL."""
+    if bootstrap_target_arch and bootstrap_target_arch.startswith("arm"):
         pattern = re.compile('/arm.*?/')
         repourl = pattern.sub("/ia32/", repourl)
         get_repo_metadata(repourl, cachedir, reponame, proxies, bootstrap_arch)
@@ -443,7 +441,7 @@ def build_bootstrap(repoList, mainrepo, cachedir, bootstrapdir, bootstrap_arch =
             if pkgurl.startswith("file://"):
                  pkgfn = pkgurl.replace("file://", "")
             else:
-                 print "Downloading %s from %s..." % (pkg, pkgurl)
+                 print "Downloading %s..." % os.path.basename(pkgurl)
                  try:
                      pkgfn = g.urlgrab(url = pkgurl, filename = pkgfn, proxies = proxies)
                  except URLGrabError, e:
@@ -470,7 +468,7 @@ def build_bootstrap(repoList, mainrepo, cachedir, bootstrapdir, bootstrap_arch =
                "make", "file", "psmisc", "satsolver-tools", "libzypp",
                "python-zypp", "mic2", "isomd5sum", "wget",
                # mtd-utils is installed to support e.g. mkfs.jffs2
-               "mtd-utils"]
+               "mtd-utils", "mtd-utils-ubi"]
     
     for pkg in pkglist:
         for node in get_my_all_deps_in_order(pkg, con):
