@@ -216,10 +216,22 @@ def set_proxy_environ():
 def unset_proxy_environ():
    if os.environ.has_key("http_proxy"):
        del os.environ["http_proxy"]
-   if os.environ.has_key("HTTP_PROXY"):
-       del os.environ["HTTP_PROXY"]
+   if os.environ.has_key("https_proxy"):
+       del os.environ["https_proxy"]
+   if os.environ.has_key("ftp_proxy"):
+       del os.environ["ftp_proxy"]
+   if os.environ.has_key("all_proxy"):
+       del os.environ["all_proxy"]
    if os.environ.has_key("no_proxy"):
        del os.environ["no_proxy"]
+   if os.environ.has_key("HTTP_PROXY"):
+       del os.environ["HTTP_PROXY"]
+   if os.environ.has_key("HTTPS_PROXY"):
+       del os.environ["HTTPS_PROXY"]
+   if os.environ.has_key("FTP_PROXY"):
+       del os.environ["FTP_PROXY"]
+   if os.environ.has_key("ALL_PROXY"):
+       del os.environ["ALL_PROXY"]
    if os.environ.has_key("NO_PROXY"):
        del os.environ["NO_PROXY"]
 
@@ -248,6 +260,7 @@ def _set_proxies(proxy = None, no_proxy = None):
                 _my_proxies[name[:-6]] = value
             else:
                 _my_noproxy = value
+
 def ip_to_int(ip):
     ipint=0
     shift=24
@@ -434,40 +447,37 @@ def get_repostr(repo, siteconf = None):
     return repostr
 
 DEFAULT_SITECONF_GLOBAL="/etc/mic2/mic2.conf"
-DEFAULT_SITECONF_USER="~/.mic2.conf"
+DEFAULT_SITECONF_USER=os.path.expanduser("~/.mic2.conf")
 
 def read_siteconf(siteconf = None):
     from ConfigParser import SafeConfigParser
 
     my_siteconf_parser = SafeConfigParser()
-    if not siteconf:
-        global_siteconf = DEFAULT_SITECONF_GLOBAL
-        if os.path.isfile(global_siteconf):
-            my_siteconf_parser.read(global_siteconf)
 
-        local_siteconf = os.path.expanduser(DEFAULT_SITECONF_USER)
-        if os.path.isfile(local_siteconf):
-            my_siteconf_parser.read(local_siteconf)
-    else:
-        my_siteconf_parser.read(siteconf)
+    siteconfigs = [DEFAULT_SITECONF_GLOBAL,DEFAULT_SITECONF_USER]
 
-    if not my_siteconf_parser.sections():
-        return None
-    else:
-        return my_siteconf_parser
+    if siteconf:
+        """ cmdline siteconfig always overrides all the others. """
+        siteconfigs = [siteconf]
+
+    readsiteconfigs = my_siteconf_parser.read(siteconfigs)
+
+    if readsiteconfigs:
+        print "Read following site configs: %s" % (siteconfigs)
+
+    return my_siteconf_parser
 
 def output_siteconf(siteconf):
-    output = ""
     if not siteconf:
-        return output
+        return None
 
+    output = ""
     for section in siteconf.sections():
         output += "[%s]\n" % section
         for option in siteconf.options(section):
             output += "%s=%s\n" % (option, siteconf.get(section, option))
         output += "\n\n"
 
-    print output
     return output
 
 def get_repostrs_from_ks(ks):
@@ -852,7 +862,7 @@ def select_ks(ksfiles):
     return ksfiles[choice-1]["filename"]
 
 
-def get_pkglist_in_group(group, patterns):
+def get_pkglist_in_patterns(group, patterns):
     found = False
     pkglist = []
     try:
@@ -888,6 +898,33 @@ def get_pkglist_in_group(group, patterns):
 
     return pkglist
 
+def get_pkglist_in_comps(group, comps):
+    found = False
+    pkglist = []
+    try:
+        root = xmlparse(comps)
+    except SyntaxError:
+        raise SyntaxError("%s syntax error." % comps)
+
+    for elm in root.getiterator("group"):
+        id = elm.find("id")
+        name = elm.find("name")
+        if id.text == group or name.text == group:
+            packagelist = elm.find("packagelist")
+            found = True
+            break
+
+    if not found:
+        return pkglist
+
+    for require in elm.getiterator("packagereq"):
+        if require.tag.endswith("packagereq"):
+            pkgname = require.text
+        if pkgname not in pkglist:
+            pkglist.append(pkgname)
+
+    return pkglist
+
 def is_statically_linked(binary):
     ret = False
     dev_null = os.open("/dev/null", os.O_WRONLY)
@@ -921,8 +958,8 @@ def setup_qemu_emulator(rootdir, arch):
     shutil.copy(qemu_emulator, rootdir + qemu_emulator)
 
     # disable selinux, selinux will block qemu emulator to run
-    if os.path.exists("/usr/sbin/setenforce"):
-        subprocess.call(["/usr/sbin/setenforce", "0"])
+    #if os.path.exists("/usr/sbin/setenforce"):
+    #   subprocess.call(["/usr/sbin/setenforce", "0"])
 
     node = "/proc/sys/fs/binfmt_misc/arm"
     if is_statically_linked(qemu_emulator) and os.path.exists(node):
